using BarContracts.BindingModels;
using BarContracts.SearchModels;
using BarContracts.ViewModels;

namespace BarContracts.BusinessLogicContracts
{
    public interface IOrderLogic
    {
        List<OrderViewModel>? ReadList(OrderSearchModel? Model);
        bool CreateOrder(OrderBindingModel Model);
        bool TakeOrderInWork(OrderBindingModel Model);
        bool FinishOrder(OrderBindingModel Model);
        bool DeliveryOrder(OrderBindingModel Model);
        OrderViewModel? ReadElement(OrderSearchModel Model);
    }
}
